﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Battleship.DrawningObjects;
using Battleship.Entities;

namespace Battleship.Generics
{
    internal class ShipCompareByColor: IComparer<DrawningShip?>
    {
        public int Compare(DrawningShip? x, DrawningShip? y)
        {
            if (x == null || x.EntityShip == null)
            {
                throw new ArgumentNullException(nameof(x));
            }
            if (y == null || y.EntityShip == null)
            {
                throw new ArgumentNullException(nameof(y));
            }
            var bodyColorCompare = x.EntityShip.BodyColor.Name.CompareTo(y.EntityShip.BodyColor.Name);
            if (bodyColorCompare != 0)
            {
                return bodyColorCompare;
            }
            if (x.EntityShip is EntityBattleship xEntityBattleship && y.EntityShip is EntityBattleship yEntityBattleship)
            {
                var BodyColorCompare = xEntityBattleship.BodyColor.Name.CompareTo(yEntityBattleship.BodyColor.Name);
                if (BodyColorCompare != 0)
                {
                    return BodyColorCompare;
                }
                var AdditionalColorCompare = xEntityBattleship.AdditionalColor.Name.CompareTo(yEntityBattleship.AdditionalColor.Name);
                if (AdditionalColorCompare != 0)
                {
                    return AdditionalColorCompare;
                }
            }
            var speedCompare = x.EntityShip.Speed.CompareTo(y.EntityShip.Speed);
            if (speedCompare != 0)
            {
                return speedCompare;
            }
            return x.EntityShip.Weight.CompareTo(y.EntityShip.Weight);
        }
    }
}
