﻿using Battleship.DrawningObjects;
using Battleship.MovementStrategy;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Battleship.Generics
{
    internal class ShipGenericCollection<T, U>
        where T : DrawningShip
        where U : IMoveableObject
    {
        private readonly int _pictureWidth;
        private readonly int _pictureHeight;
        private readonly int _placeSizeWidth = 185;
        private readonly int _placeSizeHeight = 90;
        private readonly SetGeneric<T> _collection;
        /// <summary>
        /// Получение объектов коллекции
        /// </summary>
        public IEnumerable<T?> GetShips => _collection.GetShips();
        public void Sort(IComparer<T?> comparer) => _collection.SortSet(comparer);
        public ShipGenericCollection(int picWidth, int picHeight)
        {
            int width = picWidth / _placeSizeWidth;
            int height = picHeight / _placeSizeHeight;
            _pictureWidth = picWidth;
            _pictureHeight = picHeight;
            _collection = new SetGeneric<T>(width * height);
        }
        public static bool operator +(ShipGenericCollection<T, U>? collect, T? obj)
        {
            if (obj == null)
            {
                return false;
            }
            return collect?._collection.Insert(obj, new DrawningShipEqutables()) ?? false;
        }

        public static T? operator -(ShipGenericCollection<T, U>? collect, int pos)
        {
            T? obj = collect?._collection[pos];
            if (obj != null && collect != null)
            {
                collect._collection.Remove(pos);
            }
            return obj;
        }
        public U? GetU(int pos)
        {
            return (U?)_collection[pos]?.GetMoveableObject;
        }
        public Bitmap ShowShips()
        {
            Bitmap bmp = new(_pictureWidth, _pictureHeight);
            Graphics gr = Graphics.FromImage(bmp);
            DrawBackground(gr);
            DrawObjects(gr);
            return bmp;
        }
        private void DrawBackground(Graphics g)
        {
            Pen pen = new(Color.Black, 3);
            for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
            {
                for (int j = 0; j < _pictureHeight / _placeSizeHeight +
                1; ++j)
                {
                    g.DrawLine(pen, i * _placeSizeWidth, j *
                    _placeSizeHeight, i * _placeSizeWidth + _placeSizeWidth / 2, j *
                    _placeSizeHeight);
                }
                g.DrawLine(pen, i * _placeSizeWidth, 0, i *
                _placeSizeWidth, _pictureHeight / _placeSizeHeight * _placeSizeHeight);
            }
        }
        private void DrawObjects(Graphics g)
        {
            int width = _pictureWidth/ _placeSizeWidth;
            int i = 0;
            foreach(var ship in _collection.GetShips())
            {
                if(ship != null)
                {
                    ship._pictureWidth = _pictureWidth;
                    ship._pictureHeight = _pictureHeight;
                    int inRow = _pictureWidth / _placeSizeWidth;
                    ship.SetPosition(i % inRow * _placeSizeWidth, _pictureHeight - _pictureHeight % _placeSizeHeight - (i / inRow + 1) * _placeSizeHeight + 5);
                    ship.DrawTransport(g);
                }
                i++;
            }
        }
    }
}

